(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/platform-browser'), require('@angular/core'), require('@angular/common'), require('@angular/forms')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/platform-browser', '@angular/core', '@angular/common', '@angular/forms'], factory) :
	(factory((global['handy-angular-components'] = global['handy-angular-components'] || {}),global._angular_platformBrowser,global._angular_core,global._angular_common,global._angular_forms));
}(this, (function (exports,_angular_platformBrowser,_angular_core,_angular_common,_angular_forms) { 'use strict';

var HacDropdownFilterPipe = (function () {
    function HacDropdownFilterPipe() {
    }
    /**
     * @param {?} value
     * @param {?} filter
     * @param {?=} apply
     * @return {?}
     */
    HacDropdownFilterPipe.prototype.transform = function (value, filter, apply) {
        if (apply === void 0) { apply = true; }
        if (!filter || !apply)
            return value;
        return value.filter(function (o) { return o.label.indexOf(filter) > -1 || o.key.toString() === filter; });
    };
    HacDropdownFilterPipe.decorators = [
        { type: _angular_core.Injectable },
        { type: _angular_core.Pipe, args: [{
                    name: 'hacDropdownFilter'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDropdownFilterPipe.ctorParameters = function () { return []; };
    return HacDropdownFilterPipe;
}());

var HacDropdownColumnizerPipe = (function () {
    function HacDropdownColumnizerPipe() {
    }
    /**
     * @param {?} value
     * @param {?=} columns
     * @return {?}
     */
    HacDropdownColumnizerPipe.prototype.transform = function (value, columns) {
        if (columns === void 0) { columns = 1; }
        if (typeof columns !== 'number' || columns === 1)
            return [value];
        var /** @type {?} */ output = [];
        var /** @type {?} */ currentCols = [];
        for (var /** @type {?} */ i = 1; i <= value.length; i++) {
            currentCols.push(value[i - 1]);
            if (i % columns === 0) {
                output.push(currentCols);
                currentCols = [];
            }
        }
        if (currentCols.length > 0) {
            output.push(currentCols);
        }
        return output;
    };
    HacDropdownColumnizerPipe.decorators = [
        { type: _angular_core.Injectable },
        { type: _angular_core.Pipe, args: [{
                    name: 'hacDropdownColumnizer'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDropdownColumnizerPipe.ctorParameters = function () { return []; };
    return HacDropdownColumnizerPipe;
}());

var HacDropdownComponent = (function () {
    /**
     * @param {?} elementRef
     * @param {?} dropdownFilter
     */
    function HacDropdownComponent(elementRef, dropdownFilter) {
        this.elementRef = elementRef;
        this.dropdownFilter = dropdownFilter;
        this.optionGroups = [];
        this.placeholder = 'Select';
        this.allowEmpty = false;
        this.filtrable = false;
        this.columns = 1;
        this.selectedChange = new _angular_core.EventEmitter();
        this.collapsed = true;
        this.windowHeight = 0;
        this.syncWindowHeight();
    }
    Object.defineProperty(HacDropdownComponent.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this._selected;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this._selected = v;
            var /** @type {?} */ selection = this.getSelected();
            this.filter = selection ? selection.label : null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdownComponent.prototype.onClick = function (event) {
        if (!this.elementRef.nativeElement.contains(event.target)) {
            this.closeDropdown();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdownComponent.prototype.onResize = function (event) {
        this.syncWindowHeight();
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.getSelected = function () {
        var _this = this;
        if (this.hasGroups()) {
            return this.optionGroups.map(function (f) { return f.options.find(function (o) { return o.key === _this.selected; }); }).find(function (o) { return o != null; });
        }
        return null;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    HacDropdownComponent.prototype.select = function (key) {
        this.selected = key;
        this.closeDropdown();
        this.selectedChange.emit(this.selected);
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.hasGroups = function () {
        return this.optionGroups && this.optionGroups.length > 0;
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdownComponent.prototype.openDropdown = function (e) {
        this.collapsed = false;
        if (e) {
            e.target.focus();
        }
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdownComponent.prototype.closeDropdown = function (e) {
        this.collapsed = true;
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.shouldApplyFilter = function () {
        var /** @type {?} */ selected = this.getSelected();
        return selected && this.filter !== selected.label;
    };
    /**
     * @param {?} e
     * @return {?}
     */
    HacDropdownComponent.prototype.handleKey = function (e) {
        switch (e.keyCode) {
            case 13:// Enter key
                this.selectFirstMatchingOption();
                break;
            case 27://ESC key
                this.blurFilter();
                this.closeDropdown();
                break;
        }
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.getLabelElem = function () {
        return this.elementRef.nativeElement.querySelector('.hac-dd-label');
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.calcDropdownHeight = function () {
        var /** @type {?} */ el = this.elementRef.nativeElement.querySelector('.hac-dd-list');
        var /** @type {?} */ pos = this.getPos(el);
        return window.innerHeight - 10 - pos.y + "px";
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.syncWindowHeight = function () {
        this.windowHeight = window.innerHeight;
    };
    /**
     * @param {?} el
     * @return {?}
     */
    HacDropdownComponent.prototype.getPos = function (el) {
        var /** @type {?} */ lx, /** @type {?} */ ly;
        for (lx = 0, ly = 0; el != null; lx += el.offsetLeft, ly += el.offsetTop, el = el.offsetParent) { }
        
        return { x: lx, y: ly };
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.blurFilter = function () {
        this.elementRef.nativeElement.querySelector('.hac-dd-filter').blur();
    };
    /**
     * @return {?}
     */
    HacDropdownComponent.prototype.selectFirstMatchingOption = function () {
        for (var _i = 0, _a = this.optionGroups; _i < _a.length; _i++) {
            var group = _a[_i];
            var /** @type {?} */ candidates = this.dropdownFilter.transform(group.options, this.filter);
            if (candidates.length > 0) {
                this.select(candidates[0].key);
                this.blurFilter();
                break;
            }
        }
    };
    /* Dropdown styling dimensions */
    HacDropdownComponent.decorators = [
        { type: _angular_core.Component, args: [{
                    selector: 'hac-dropdown',
                    template: "<div class=\"hac-dd\"> <div class=\"hac-dd-label\"  (click)=\"openDropdown($event)\"> <input  type=\"text\" class=\"hac-dd-filter\" [placeholder]=\"placeholder\" id=\"{{id}}\" [(ngModel)]=\"filter\" [readonly]=\"!filtrable\" (keyup)=\"handleKey($event)\" (focus)=\"openDropdown($event)\" (focusOut)=\"closeDropdown($event)\" /> <i class=\"hac-dd-caret hac-dd-caret--down\" aria-hidden=\"true\"></i> </div> <div class=\"hac-dd-list\"  [ngClass]=\"{ 'hac-dd-list--is-collapsed': collapsed }\"  *ngIf=\"hasGroups()\" [hacCopyWidth]=\"getLabelElem()\" widthCssProperty=\"width\" [skip]=\"collapsed\" [style.maxHeight]=\"calcDropdownHeight()\"> <div class=\"hac-dd-option\" *ngIf=\"allowEmpty\" (click)=\"select(null)\">{{placeholder}}</div> <div class=\"hac-dd-group\" *ngFor=\"let group of optionGroups\"> <div class=\"hac-dd-grouplabel\" *ngIf=\"group.label\">{{group.label}}</div> <div class=\"hac-dd-optgroup\"> <div class=\"hac-dd-optrow\" *ngFor=\"let optionRow of group.options | hacDropdownFilter: filter : shouldApplyFilter() | hacDropdownColumnizer: columns\" [ngClass]=\"{'hac-dd-optrow--single-col': columns === 1}\"> <div *ngFor=\"let option of optionRow\"  (click)=\"select(option.key)\" class=\"hac-dd-option hac-dd-choice\"  [ngClass]=\"{  'hac-dd-option--is-selected': option.key === selected }\"> {{option.label}} </div> </div> </div> </div> </div> </div>",
                    providers: [
                        HacDropdownFilterPipe,
                        HacDropdownColumnizerPipe
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDropdownComponent.ctorParameters = function () { return [
        { type: _angular_core.ElementRef, },
        { type: HacDropdownFilterPipe, },
    ]; };
    HacDropdownComponent.propDecorators = {
        'optionGroups': [{ type: _angular_core.Input },],
        'placeholder': [{ type: _angular_core.Input },],
        'allowEmpty': [{ type: _angular_core.Input },],
        'filtrable': [{ type: _angular_core.Input },],
        'columns': [{ type: _angular_core.Input },],
        'selectedChange': [{ type: _angular_core.Output },],
        'id': [{ type: _angular_core.Input },],
        'selected': [{ type: _angular_core.Input },],
        'onClick': [{ type: _angular_core.HostListener, args: ['document:click', ['$event'],] },],
        'onResize': [{ type: _angular_core.HostListener, args: ['window:resize', ['$event'],] },],
    };
    return HacDropdownComponent;
}());

var DateHelper = (function () {
    function DateHelper() {
    }
    /**
     * Format date to 'yyyy-MM-dd' format
    \@param day
     * @param {?} day
     * @return {?}
     */
    DateHelper.formatIsoDate = function (day) {
        var /** @type {?} */ d = this.ensureDateObject(day);
        var /** @type {?} */ monthNum = d.getMonth() + 1;
        return d.getFullYear() + "-" + (monthNum < 10 ? '0' : '') + monthNum + "-" + (d.getDate() < 10 ? '0' : '') + d.getDate();
    };
    /**
     * Verify if both dates are the same day
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.areDatesEqual = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) === this.formatIsoDate(day2);
    };
    /**
     * Checks if candidate is in range between startRange and endRange including both limits
    \@param candidate
    \@param startRange
    \@param endRange
     * @param {?} candidate
     * @param {?} startRange
     * @param {?} endRange
     * @return {?}
     */
    DateHelper.isInRange = function (candidate, startRange, endRange) {
        if (!candidate || !startRange || !endRange)
            return false;
        var /** @type {?} */ candidateFormatted = this.formatIsoDate(candidate);
        return candidateFormatted >= this.formatIsoDate(startRange) && candidateFormatted <= this.formatIsoDate(endRange);
    };
    /**
     * Checks if day1 is greater than day 2
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.isGreater = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) > this.formatIsoDate(day2);
    };
    /**
     * Checks if day1 is greater or equal than day 2
    \@param day1
    \@param day2
     * @param {?} day1
     * @param {?} day2
     * @return {?}
     */
    DateHelper.isGreaterOrEqual = function (day1, day2) {
        return day1 && day2 && this.formatIsoDate(day1) >= this.formatIsoDate(day2);
    };
    /**
     * Get today date at 00:00h
     * @return {?}
     */
    DateHelper.today = function () {
        var /** @type {?} */ today = new Date();
        this.resetTime(today);
        return today;
    };
    /**
     * Resets time part from Date object to 00:00:00.000h
     * @param {?} date
     * @return {?}
     */
    DateHelper.resetTime = function (date) {
        if (typeof date === 'string') {
            date = this.formatIsoDate(date);
            return;
        }
        date.setHours(0);
        date.setMinutes(0);
        date.setSeconds(0);
        date.setMilliseconds(0);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    DateHelper.ensureDateObject = function (date) {
        return typeof date === 'string' ? new Date(date) : date;
    };
    return DateHelper;
}());

/*
 <  January 2017   >
_____________________
[M][T][W][T][F][S][S]
---------------------
 _  1  2  3  4  5  6
 7  8  9  10 11 12 13
 14 15 16 17 18 19 20
 21 22 23 24 25 26 27
 28 29 30 31 __ __ __

*/
var WeekDay = {};
WeekDay.Sunday = 0;
WeekDay.Monday = 1;
WeekDay.Tuesday = 2;
WeekDay.Wednesday = 3;
WeekDay.Thursday = 4;
WeekDay.Friday = 5;
WeekDay.Saturday = 6;
WeekDay[WeekDay.Sunday] = "Sunday";
WeekDay[WeekDay.Monday] = "Monday";
WeekDay[WeekDay.Tuesday] = "Tuesday";
WeekDay[WeekDay.Wednesday] = "Wednesday";
WeekDay[WeekDay.Thursday] = "Thursday";
WeekDay[WeekDay.Friday] = "Friday";
WeekDay[WeekDay.Saturday] = "Saturday";
var weekDayList = [
    WeekDay.Monday,
    WeekDay.Tuesday,
    WeekDay.Wednesday,
    WeekDay.Thursday,
    WeekDay.Friday,
    WeekDay.Saturday,
    WeekDay.Sunday
];
var startWeekDay = WeekDay.Monday;
var HacCalendarDayModel = (function () {
    /**
     * @param {?} day
     */
    function HacCalendarDayModel(day) {
        this.day = DateHelper.ensureDateObject(day);
    }
    /**
     * @return {?}
     */
    HacCalendarDayModel.prototype.isToday = function () {
        return DateHelper.areDatesEqual(this.day, new Date());
    };
    return HacCalendarDayModel;
}());
var HacCalendarWeekModel = (function () {
    function HacCalendarWeekModel() {
        this.days = [];
    }
    /**
     * @param {?} day
     * @return {?}
     */
    HacCalendarWeekModel.prototype.addDay = function (day) {
        if (this.days.length === 0 && day.getDay() !== startWeekDay) {
            var /** @type {?} */ previousDay = new Date(day);
            previousDay.setDate(previousDay.getDate() - 1);
            this.addDay(previousDay);
        }
        this.days.push(new HacCalendarDayModel(day));
    };
    return HacCalendarWeekModel;
}());
var HacCalendarModel = (function () {
    /**
     * @param {?} month
     */
    function HacCalendarModel(month) {
        this.weeks = [];
        this.month = new Date(month.getFullYear(), month.getMonth(), 1, 0, 0, 0, 0);
        this.buildCalendar();
    }
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.buildCalendar = function () {
        var /** @type {?} */ day = new Date(this.month);
        var /** @type {?} */ lastDay = this.getMonthLastDay();
        while (day <= lastDay) {
            // add new week when on first week day or first week
            if (day.getDay() === startWeekDay || this.weeks.length === 0) {
                var /** @type {?} */ week = new HacCalendarWeekModel();
                week.addDay(new Date(day));
                this.weeks.push(week);
            }
            else {
                this.weeks[this.weeks.length - 1].addDay(new Date(day));
            }
            day.setDate(day.getDate() + 1); // next day
        }
        // add remaining days of last week from next month
        var /** @type {?} */ lastWeek = this.weeks[this.weeks.length - 1];
        while (lastWeek.days.length < 7) {
            lastWeek.addDay(new Date(day));
            day.setDate(day.getDate() + 1);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacCalendarModel.prototype.contains = function (day) {
        return day.day.getFullYear() === this.month.getFullYear() && day.day.getMonth() === this.month.getMonth();
    };
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.getMonthLastDay = function () {
        return new Date(this.month.getFullYear(), this.month.getMonth() + 1, 0);
    };
    /**
     * @return {?}
     */
    HacCalendarModel.prototype.getPreviousMonthLastDay = function () {
        return new Date(this.month.getFullYear(), this.month.getMonth(), 0);
    };
    return HacCalendarModel;
}());

var HacWeekDayFormatter = (function () {
    /**
     * @param {?} datePipe
     */
    function HacWeekDayFormatter(datePipe) {
        this.datePipe = datePipe;
        this.Monday = new Date(2016, 1, 1, 0, 0, 0, 0);
        this.Tuesday = new Date(2016, 1, 2, 0, 0, 0, 0);
        this.Wednesday = new Date(2016, 1, 3, 0, 0, 0, 0);
        this.Thursday = new Date(2016, 1, 4, 0, 0, 0, 0);
        this.Friday = new Date(2016, 1, 5, 0, 0, 0, 0);
        this.Saturday = new Date(2016, 1, 6, 0, 0, 0, 0);
        this.Sunday = new Date(2016, 1, 7, 0, 0, 0, 0);
    }
    /**
     * @param {?} value
     * @param {?=} customFormat
     * @return {?}
     */
    HacWeekDayFormatter.prototype.transform = function (value, customFormat) {
        if (customFormat === void 0) { customFormat = 'EEE'; }
        if (value != null) {
            switch (value) {
                case WeekDay.Monday:
                    return this.datePipe.transform(this.Monday, customFormat);
                case WeekDay.Tuesday:
                    return this.datePipe.transform(this.Tuesday, customFormat);
                case WeekDay.Wednesday:
                    return this.datePipe.transform(this.Wednesday, customFormat);
                case WeekDay.Thursday:
                    return this.datePipe.transform(this.Thursday, customFormat);
                case WeekDay.Friday:
                    return this.datePipe.transform(this.Friday, customFormat);
                case WeekDay.Saturday:
                    return this.datePipe.transform(this.Saturday, customFormat);
                case WeekDay.Sunday:
                    return this.datePipe.transform(this.Sunday, customFormat);
            }
        }
        return '';
    };
    HacWeekDayFormatter.decorators = [
        { type: _angular_core.Injectable },
        { type: _angular_core.Pipe, args: [{
                    name: 'hacWeekDayFormatter'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacWeekDayFormatter.ctorParameters = function () { return [
        { type: _angular_common.DatePipe, },
    ]; };
    return HacWeekDayFormatter;
}());

var HacImmutableHelper = (function () {
    function HacImmutableHelper() {
    }
    /**
     * @template T
     * @param {?} obj
     * @param {?} updatedProperties
     * @return {?}
     */
    HacImmutableHelper.set = function (obj, updatedProperties) {
        return Object.assign({}, obj, updatedProperties);
    };
    return HacImmutableHelper;
}());

var HacDatepickerComponent = (function () {
    /**
     * @param {?} elementRef
     */
    function HacDatepickerComponent(elementRef) {
        this.elementRef = elementRef;
        this.startDateChange = new _angular_core.EventEmitter();
        this.endDateChange = new _angular_core.EventEmitter();
        this._options = {};
        this.calendars = [];
        this.weekDayList = weekDayList;
        this._collapsed = true;
    }
    Object.defineProperty(HacDatepickerComponent.prototype, "startDate", {
        /**
         * @return {?}
         */
        get: function () {
            return this._startDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            var _this = this;
            this._startDate = v;
            if (!v)
                return;
            // reset invalid dates (disabled day or past time)
            if (this.isDisabled(new HacCalendarDayModel(v))
                || (this.options.range && this._endDate && DateHelper.isGreater(v, this._endDate))) {
                setTimeout(function () { return _this.setStartDate(null); }, 0);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "endDate", {
        /**
         * @return {?}
         */
        get: function () {
            return this._endDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            var _this = this;
            this._endDate = v;
            if (!v)
                return;
            // reset invalid dates (disabled day or past time)
            if (this.isDisabled(new HacCalendarDayModel(v))
                || (this.options.range && this._startDate && DateHelper.isGreater(this._startDate, v))) {
                setTimeout(function () { return _this.setEndDate(null); }, 0);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this.startDate;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this.setStartDate(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "selectedChange", {
        /**
         * @return {?}
         */
        get: function () {
            return this.startDateChange;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "options", {
        /**
         * @return {?}
         */
        get: function () {
            return this._options;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this._options = v;
            this.setOptionsDefaults();
            this.buildCalendarModel();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HacDatepickerComponent.prototype, "collapsed", {
        /**
         * @return {?}
         */
        get: function () {
            return this._collapsed;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this.animate();
            this._collapsed = v;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.ngOnInit = function () {
        this.setOptionsDefaults();
        this.buildCalendarModel();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDatepickerComponent.prototype.onClick = function (event) {
        if (this.options.elementId && event.target.htmlFor === this.options.elementId) {
            this.collapsed = false;
        }
        else if (!this.elementRef.nativeElement.contains(event.target)) {
            this.collapsed = true;
        }
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.buildCalendarModel = function () {
        this.calendars = [];
        var /** @type {?} */ currentMonth = new Date(this.options.currentDisplayMonth);
        var /** @type {?} */ calendarMonths = this.options.showMonths > 0 ? this.options.showMonths : 1;
        for (var /** @type {?} */ i = 0; i < calendarMonths; i++) {
            this.calendars.push(new HacCalendarModel(currentMonth));
            currentMonth.setMonth(currentMonth.getMonth() + 1);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.select = function (day) {
        if (this.isDisabled(day)) {
            return;
        }
        if (!this.options.range || this.getSelectionKind() === 'start') {
            this.setStartDate(day.day);
            if (!this.forcedSelectionKind) {
                this.setEndDate(null);
            }
            else if (this.options.range) {
                this.forceSelectionKind('end');
                if (day.day > this._endDate) {
                    this.setEndDate(null);
                }
            }
            this.collapsed = !this.options.range;
        }
        else {
            this.setEndDate(day.day);
            this.collapsed = true;
            this.forceSelectionKind(null);
        }
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isSelected = function (day) {
        return DateHelper.areDatesEqual(day.day, this._startDate)
            || this.options.range && DateHelper.areDatesEqual(day.day, this._endDate);
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isDisabled = function (day) {
        var /** @type {?} */ exceedsMinDate = this.options.minDate && DateHelper.isGreater(this.options.minDate, day.day);
        if (exceedsMinDate)
            return true;
        var /** @type {?} */ exceedsMaxDate = this.options.maxDate && DateHelper.isGreater(day.day, this.options.maxDate);
        if (exceedsMaxDate)
            return true;
        var /** @type {?} */ isBlacklisted = this.options.dayList && this.options.dayListKind === 'blacklist' && this.isInDayList(day);
        isBlacklisted = isBlacklisted || this.options.dayList && this.options.dayListKind === 'whitelist' && !this.isInDayList(day);
        if (isBlacklisted)
            return true;
        var /** @type {?} */ isPastOverflow = this.getSelectionKind() === 'end' && DateHelper.isGreater(this._startDate, day.day);
        return isPastOverflow;
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isStart = function (day) {
        return this.options.range && DateHelper.areDatesEqual(day.day, this._startDate);
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isEnd = function (day) {
        return this.options.range && DateHelper.areDatesEqual(day.day, this._endDate);
    };
    /**
     * @param {?} day
     * @param {?=} useHover
     * @return {?}
     */
    HacDatepickerComponent.prototype.inRange = function (day, useHover) {
        if (useHover === void 0) { useHover = false; }
        if (!this.options.range)
            return false;
        var /** @type {?} */ startDate = useHover ? this.getFirstRangeDay() : this._startDate;
        var /** @type {?} */ endDate = useHover ? this.getLastRangeDay() : this._endDate;
        if (!startDate || !endDate)
            return false;
        return this.options.range && DateHelper.isInRange(day.day, startDate, endDate);
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setHoverDate = function (day) {
        this.hoverDate = day.day;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.clearDates = function () {
        this.setStartDate(null);
        this.setEndDate(null);
    };
    /**
     * @param {?=} $event
     * @return {?}
     */
    HacDatepickerComponent.prototype.prevMonth = function ($event) {
        this.options.currentDisplayMonth.setMonth(this.options.currentDisplayMonth.getMonth() - 1);
        this.options = HacImmutableHelper.set(this.options, {
            currentDisplayMonth: this.options.currentDisplayMonth
        });
        this.buildCalendarModel();
        this.animate();
        $event.stopPropagation();
    };
    /**
     * @param {?=} $event
     * @return {?}
     */
    HacDatepickerComponent.prototype.nextMonth = function ($event) {
        this.options.currentDisplayMonth.setMonth(this.options.currentDisplayMonth.getMonth() + 1);
        this.options = HacImmutableHelper.set(this.options, {
            currentDisplayMonth: this.options.currentDisplayMonth
        });
        this.buildCalendarModel();
        this.animate();
        $event.stopPropagation();
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.toggleCalendar = function () {
        this.collapsed = !this.collapsed;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getSelectionKind = function () {
        if (this.forcedSelectionKind) {
            return this.forcedSelectionKind;
        }
        return !this._startDate || (this.options.range && this._startDate && this._endDate) ? 'start' : 'end';
    };
    /**
     * @param {?=} kind
     * @return {?}
     */
    HacDatepickerComponent.prototype.forceSelectionKind = function (kind) {
        this.forcedSelectionKind = kind;
        if (kind) {
            this.collapsed = false;
        }
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.selectToday = function () {
        var /** @type {?} */ today = new HacCalendarDayModel(DateHelper.today());
        this.select(today);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.isTodayActionEnabled = function () {
        return !this.isDisabled(new HacCalendarDayModel(DateHelper.today()));
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getSelectorElem = function () {
        return this.elementRef.nativeElement.querySelector('.hac-cal-selectorwrapper');
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getCalendarWidthMode = function () {
        return this.options.useSelectorWidth ? 'width' : 'minWidth';
    };
    /**
     * @param {?} calendarIndex
     * @return {?}
     */
    HacDatepickerComponent.prototype.isPrevArrowVisible = function (calendarIndex) {
        var /** @type {?} */ isInRange = !this.getMinDate() || (this.getMinDate() &&
            DateHelper.isGreaterOrEqual(this.calendars[calendarIndex].getPreviousMonthLastDay(), this.getMinDate()));
        return calendarIndex === 0 && isInRange;
    };
    /**
     * @param {?} calendarIndex
     * @return {?}
     */
    HacDatepickerComponent.prototype.isNextArrowVisible = function (calendarIndex) {
        var /** @type {?} */ isInRange = !this.getMaxDate() || (this.getMaxDate() &&
            DateHelper.isGreaterOrEqual(this.getMaxDate(), this.calendars[calendarIndex].getMonthLastDay()));
        return (calendarIndex === this.calendars.length - 1) && isInRange;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.setOptionsDefaults = function () {
        var /** @type {?} */ startDate = this._options.currentDisplayMonth ?
            new Date(this.options.currentDisplayMonth) :
            this._startDate ? DateHelper.ensureDateObject(this._startDate) : new Date();
        if ((this._options.minDate && DateHelper.isGreater(this._options.minDate, startDate)) ||
            (this._options.maxDate && DateHelper.isGreater(startDate, this._options.maxDate))) {
            startDate = this._options.minDate ? this._options.minDate : this._options.maxDate;
        }
        var /** @type {?} */ currentDisplayMonth = new Date(startDate);
        currentDisplayMonth.setDate(1);
        DateHelper.resetTime(currentDisplayMonth);
        this._options.currentDisplayMonth = currentDisplayMonth;
        this._options.startDatePlaceholder = this._options.startDatePlaceholder ? this._options.startDatePlaceholder : 'Select';
        this._options.endDatePlaceholder = this._options.endDatePlaceholder ? this._options.endDatePlaceholder : 'Select';
        this._options.startDateFormat = this._options.startDateFormat ? this._options.startDateFormat : 'mediumDate';
        this._options.endDateFormat = this._options.endDateFormat ? this._options.endDateFormat : 'mediumDate';
        this._options.range = this._options.range || false;
        this._options.showMonths = this._options.showMonths || 1;
        this._options.dayListKind = this._options.dayListKind || 'blacklist';
        this._options.enableTodayAction = this._options.enableTodayAction || false;
        this._options.todayActionLabel = this._options.todayActionLabel || 'Today';
        this._options.useSelectorWidth = this._options.useSelectorWidth || false;
        // Force triggering of new selections to ensure that are still valid dates
        this.setStartDate(this._startDate);
        this.setEndDate(this._endDate);
    };
    /**
     * @param {?=} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setStartDate = function (day) {
        this.startDate = day;
        this.startDateChange.emit(this._startDate);
        this.hoverDate = DateHelper.ensureDateObject(this._startDate);
    };
    /**
     * @param {?=} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.setEndDate = function (day) {
        this.endDate = day;
        this.endDateChange.emit(this._endDate);
        this.hoverDate = DateHelper.ensureDateObject(this._endDate);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getFirstRangeDay = function () {
        if (this.getSelectionKind() === 'end') {
            return this._startDate;
        }
        return this.hoverDate || this._startDate;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getLastRangeDay = function () {
        if (this.getSelectionKind() === 'start') {
            return this._endDate;
        }
        return this.hoverDate || this._endDate;
    };
    /**
     * @param {?} day
     * @return {?}
     */
    HacDatepickerComponent.prototype.isInDayList = function (day) {
        if (!this.options.dayList)
            return false;
        var /** @type {?} */ year = this.options.dayList[day.day.getFullYear()];
        if (!year)
            return false;
        var /** @type {?} */ month = year[day.day.getMonth() + 1];
        if (!month)
            return false;
        return month.indexOf(day.day.getDate()) > -1;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.animate = function () {
        var _this = this;
        var /** @type {?} */ animatingCssClass = 'hac-cal-wrapper--animating';
        this.elementRef.nativeElement.querySelector('.hac-cal-wrapper').classList.remove(animatingCssClass);
        setTimeout(function () { return _this.elementRef.nativeElement.querySelector('.hac-cal-wrapper').classList.add(animatingCssClass); }, 100);
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getMinDate = function () {
        return this._options.minDate;
    };
    /**
     * @return {?}
     */
    HacDatepickerComponent.prototype.getMaxDate = function () {
        return this._options.maxDate;
    };
    HacDatepickerComponent.decorators = [
        { type: _angular_core.Component, args: [{
                    selector: 'hac-datepicker',
                    template: "<div class=\"hac-cal\"> <!-- selector wrapper --> <div class=\"hac-cal-selectorwrapper\"> <input type=\"text\" style=\"display: none;\" *ngIf=\"options.elementId\" id=\"{{options.elementId}}\" (focus)=\"forceSelectionKind('start')\"> <!-- cal icon selector --> <span class=\"hac-cal-icon\" (click)=\"toggleCalendar()\"></span> <!-- start/single date --> <span class=\"hac-cal-date hac-cal-startdate\" (click)=\"forceSelectionKind('start')\" [ngClass]=\"{'hac-cal-date--is-active': getSelectionKind() === 'start' && !collapsed }\"> <span *ngIf=\"startDate\"> {{startDate | date: options?.startDateFormat}} </span> <span *ngIf=\"!startDate\">{{options?.startDatePlaceholder}}</span> </span> <!-- end date --> <span class=\"hac-cal-date hac-cal-enddate\" *ngIf=\"options?.range\" (click)=\"forceSelectionKind('end')\" [ngClass]=\"{'hac-cal-date--is-active': getSelectionKind() === 'end' && !collapsed }\"> <span *ngIf=\"endDate\"> {{endDate | date: options?.endDateFormat}} </span> <span *ngIf=\"!endDate\">{{options?.endDatePlaceholder}}</span> </span> <span class=\"hac-cal-cross\" (click)=\"clearDates()\"></span> </div> <!-- calendar wrapper --> <div class=\"hac-cal-wrapper\"  [ngClass]=\"{ 'hac-cal-wrapper--is-opened': !collapsed }\" [hacCopyWidth]=\"getSelectorElem()\" [widthCssProperty]=\"getCalendarWidthMode()\" [skip]=\"collapsed\"> <div *ngFor=\"let calendar of calendars; let i = index\"  class=\"hac-cal-month\" [ngClass]=\"{ 'hac-cal-separator': i > 0 && i < calendars.length }\"> <!-- calendar header --> <div class=\"hac-cal-header\"> <div class=\"hac-cal-arrow\"> <span *ngIf=\"isPrevArrowVisible(i)\" (click)=\"prevMonth($event)\" class=\"hac-cal-chevron hac-cal-chevron--left\"></span> </div> <strong class=\"hac-cal-monthlabel\"> <span class=\"hac-cal-monthlabel--year\">{{calendar.month | date : 'y '}}</span> <span class=\"hac-cal-monthlabel--month\">{{calendar.month | date : 'MMMM'}}</span> </strong> <div class=\"hac-cal-arrow\"> <span *ngIf=\"isNextArrowVisible(i)\" (click)=\"nextMonth($event)\" class=\"hac-cal-chevron hac-cal-chevron--right\"></span> </div> </div> <!-- week day list --> <div class=\"hac-cal-weedays\"> <div *ngFor=\"let weekDay of weekDayList\" class=\"hac-cal-weekday\"> {{ weekDay | hacWeekDayFormatter }} </div> </div> <!-- weeks --> <div class=\"hac-cal-weeks\"> <div *ngFor=\"let week of calendar.weeks\" class=\"hac-cal-week\"> <div *ngFor=\"let day of week.days\"  class=\"hac-cal-day\" (click)=\"select(day)\" (mouseover)=\"setHoverDate(day)\" [ngClass]=\"{  'hac-cal-day--is-selected': isSelected(day), 'hac-cal-day--is-hidden': !calendar.contains(day), 'hac-cal-day--is-disabled': isDisabled(day), 'hac-cal-day--is-today': day.isToday(), 'hac-cal-day--is-start': isStart(day), 'hac-cal-day--is-end': isEnd(day), 'hac-cal-day--in-range': inRange(day), 'hac-cal-day--in-rangehover': inRange(day, true) }\"> <div class=\"hac-cal-daynum\"  title=\"{{ day?.day | date : 'fullDate'}}\"> {{ day?.day | date : 'd'}} </div> </div> </div> </div> <!-- today action --> <div *ngIf=\"i === 0 && isTodayActionEnabled()\"> <button class=\"hac-cal-todaybtn\" type=\"button\" (click)=\"selectToday()\">{{options.todayActionLabel}}</button> </div> </div> </div> </div>",
                    providers: [
                        _angular_common.DatePipe,
                        HacWeekDayFormatter
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacDatepickerComponent.ctorParameters = function () { return [
        { type: _angular_core.ElementRef, },
    ]; };
    HacDatepickerComponent.propDecorators = {
        'startDate': [{ type: _angular_core.Input },],
        'endDate': [{ type: _angular_core.Input },],
        'startDateChange': [{ type: _angular_core.Output },],
        'endDateChange': [{ type: _angular_core.Output },],
        'selected': [{ type: _angular_core.Input },],
        'selectedChange': [{ type: _angular_core.Output },],
        'options': [{ type: _angular_core.Input },],
        'onClick': [{ type: _angular_core.HostListener, args: ['document:click', ['$event'],] },],
    };
    return HacDatepickerComponent;
}());

var HacCopyWidthDirective = (function () {
    /**
     * @param {?} elementRef
     */
    function HacCopyWidthDirective(elementRef) {
        this.elementRef = elementRef;
        this.widthCssProperty = 'width';
        this.skip = false;
    }
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.ngOnInit = function () {
        this.calcWidth();
    };
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.ngDoCheck = function () {
        this.calcWidth();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacCopyWidthDirective.prototype.onResize = function (event) {
        this.calcWidth();
    };
    /**
     * @return {?}
     */
    HacCopyWidthDirective.prototype.calcWidth = function () {
        if (this.skip) {
            this.setWidth(null);
            return;
        }
        if (!this.hacCopyWidth) {
            this.setWidth(0);
            return;
        }
        var /** @type {?} */ computedStyle = getComputedStyle(this.hacCopyWidth, null);
        var /** @type {?} */ borderLeft = parseFloat(computedStyle.borderLeftWidth.replace('px', ''));
        var /** @type {?} */ borderRight = parseFloat(computedStyle.borderRightWidth.replace('px', ''));
        var /** @type {?} */ finalWidth = this.hacCopyWidth.offsetWidth - borderLeft - borderRight;
        this.setWidth(finalWidth);
    };
    /**
     * @param {?} width
     * @return {?}
     */
    HacCopyWidthDirective.prototype.setWidth = function (width) {
        this.elementRef.nativeElement.style[this.widthCssProperty] = width != null ? width + "px" : null;
    };
    HacCopyWidthDirective.decorators = [
        { type: _angular_core.Directive, args: [{
                    selector: '[hacCopyWidth]'
                },] },
    ];
    /**
     * @nocollapse
     */
    HacCopyWidthDirective.ctorParameters = function () { return [
        { type: _angular_core.ElementRef, },
    ]; };
    HacCopyWidthDirective.propDecorators = {
        'hacCopyWidth': [{ type: _angular_core.Input },],
        'widthCssProperty': [{ type: _angular_core.Input },],
        'skip': [{ type: _angular_core.Input },],
        'onResize': [{ type: _angular_core.HostListener, args: ['window:resize', ['$event'],] },],
    };
    return HacCopyWidthDirective;
}());

var HacModule = (function () {
    function HacModule() {
    }
    /**
     * @return {?}
     */
    HacModule.forRoot = function () {
        return {
            ngModule: HacModule
        };
    };
    HacModule.decorators = [
        { type: _angular_core.NgModule, args: [{
                    declarations: [
                        HacDropdownComponent,
                        HacDropdownFilterPipe,
                        HacDropdownColumnizerPipe,
                        HacDatepickerComponent,
                        HacWeekDayFormatter,
                        HacCopyWidthDirective
                    ],
                    imports: [
                        _angular_platformBrowser.BrowserModule,
                        _angular_forms.FormsModule,
                        _angular_common.CommonModule
                    ],
                    exports: [
                        HacDropdownComponent,
                        HacDatepickerComponent,
                        HacCopyWidthDirective
                    ]
                },] },
    ];
    /**
     * @nocollapse
     */
    HacModule.ctorParameters = function () { return []; };
    return HacModule;
}());

exports.HacDatepickerComponent = HacDatepickerComponent;
exports.HacWeekDayFormatter = HacWeekDayFormatter;
exports.HacDropdownComponent = HacDropdownComponent;
exports.HacDropdownFilterPipe = HacDropdownFilterPipe;
exports.HacDropdownColumnizerPipe = HacDropdownColumnizerPipe;
exports.HacCopyWidthDirective = HacCopyWidthDirective;
exports.HacModule = HacModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
